@@ -156,7 +156,7 @@ def get_apr(date, pct_change, volume_df = None, current_apr=None):
     current_apr_ethbtc = apr/100.
     daily_current_apr_ethbtc= (1.+current_apr_ethbtc)**(1./365.)-1.
 
-    ratio = volume_df.loc[date].volumeToken1 / volume_df.iloc[-1].volumeToken1
+    ratio = volume_df.loc[date].volumeToken0 / volume_df.iloc[-1].volumeToken0
     prorata_daily_current_apr_ethbtc = daily_current_apr_ethbtc * ratio
     return prorata_daily_current_apr_ethbtc
 
@@ -177,7 +177,9 @@ def get_swapping_cost(swapping_size_proportion =None, swapping_cost_map=None):
         print(e)
     return cost
 
-def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('ETH', 'BTC'), bb_periods=[5, 15, 30, 55], stdNbrs=[2., 3., 5.], distance_to_bound_percs = [0,5,10,15,20,30,40], current_apr=None, bypass_signal=True, bypass_stoploss=True, swapping_cost_map = {1:1e-4, 10:0.01, 50:0.05, 75:0.075, 100:0.1}):
+
+
+def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('ETH', 'BTC'), bb_periods=[5, 15, 30, 55], stdNbrs=[2., 3., 5.], distance_to_bound_percs = [0,5,10,15,20,30,40], current_apr=None, bypass_signal=True, bypass_stoploss=True, swapping_cost_map = {1:1e-4, 10:0.01, 50:0.05, 75:0.075, 100:0.1},starting_number_of_numeraire = 1000):
     ssj = me_pair[0]
     ssj_against = me_pair[1]
     objectives = []
@@ -396,6 +398,7 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
 
                     stages.append({
                         'date': row['Date'],
+                        'Close':row['Close'],
                         'incurred_v3': ILv3,
                         'incurred_v2': ILv2,
                         'pct_change' : pct_change,
@@ -437,22 +440,22 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                 ####### ####### ####### ####### swapping cost analysis
                 ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### #######
                 ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### #######
-
                 def rebalancing_signal(row):
                     we_rebalance_signal = row['we_rebalance_signal']
                     we_rebalance_bound = row['we_rebalance_bounds']
                     return we_rebalance_bound or we_rebalance_signal
 
-
                 stages_df['rebalancing'] = stages_df.apply(rebalancing_signal, axis=1)
-                rebalancing_df = stages_df[stages_df['rebalancing']]
+
+                rebalancing_df = stages_df[stages_df['rebalancing']].copy()
+
                 if len(rebalancing_df)>0:
                     rebalancing_df['leaving_initial_price'] = rebalancing_df['initial_price'].shift(+1)
-                    rebalancing_df['leaving_initial_price'].iloc[0]=begining_initial_price
+                    rebalancing_df['leaving_initial_price'].iloc[0] = begining_initial_price
                     rebalancing_df['leaving_real_upper_bound'] = rebalancing_df['real_upper_bound'].shift(+1)
                     rebalancing_df['leaving_real_lower_bound'] = rebalancing_df['real_lower_bound'].shift(+1)
-                    rebalancing_df['leaving_real_upper_bound'].iloc[0]=initial_real_upper_bound
-                    rebalancing_df['leaving_real_lower_bound'].iloc[0]=initial_real_lower_bound
+                    rebalancing_df['leaving_real_upper_bound'].iloc[0] = initial_real_upper_bound
+                    rebalancing_df['leaving_real_lower_bound'].iloc[0] = initial_real_lower_bound
 
                     rebal_data_df = rebalancing_df[['date','leaving_initial_price', 'initial_price', 'real_upper_bound', 'real_lower_bound', 'leaving_real_upper_bound', 'leaving_real_lower_bound']]
 
@@ -465,7 +468,6 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                     ### ether = X (risqué)
                     ### BTC = Y (numéraire)
                     ### P = P ether in BTC
-
                     def get_liquidity_from_BTC(nb_amount_BTC, current_price, lower_bound ):
                         return nb_amount_BTC/(np.sqrt(current_price) - np.sqrt(lower_bound) )
 
@@ -478,7 +480,7 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                     def get_btc_from_liquidity(liquidity, current_price, lower_bound):
                         return liquidity*(np.sqrt(current_price) - np.sqrt(lower_bound) )
 
-                    starting_number_of_bitcoins = 10
+
 
 
                     previous_liquidities = np.zeros(len(rebal_data_df))
@@ -500,10 +502,14 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                     new_eth = np.nan
 
                     rebal_data_df.index = range(len(rebal_data_df))
+                    to_keep_init_liq = np.nan
+                    to_keep_init_eth = np.nan
+                    to_keep_init_btc = np.nan
+                    to_keep_init_price = np.nan
                     for ii, row in rebal_data_df.iterrows():
                         if ii == 0 :
                             ### btc at initial date
-                            previous_btc = starting_number_of_bitcoins
+                            previous_btc = starting_number_of_numeraire
                             ## initial position configuration: leaving_initial_price inbetween leaving_real_upper_bound and leaving_real_lower_bound
                             leaving_initial_price = row['leaving_initial_price']
                             leaving_real_upper_bound = row['leaving_real_upper_bound']
@@ -513,11 +519,17 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                             ## ether at initial date
                             previous_eth = get_ether_from_liquidity(previous_liquidity, leaving_initial_price,leaving_real_upper_bound)
                             previous_liquidity_bis = get_liquidity_from_ETH(previous_eth, leaving_initial_price,leaving_real_upper_bound)
+                            to_keep_init_liq=previous_liquidity
+                            to_keep_init_eth=previous_eth
+                            to_keep_init_btc = previous_btc
+                            to_keep_init_price = leaving_initial_price
+
                             assert abs(previous_liquidity_bis - previous_liquidity) < 1e-3
                         else :
                             previous_btc = new_btc
                             previous_eth = new_eth
                             previous_liquidity = new_liquidity
+
                         swapping_size = np.nan
                         # we leave the 'leaving position' at a new price which will become the new initial price
                         newly_initial_price = row['initial_price']
@@ -548,19 +560,45 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                         new_real_lower_bound = row['real_lower_bound']
                         new_real_upper_bound = row['real_upper_bound']
                         # rebalancing the bounds
-                        if deltaBTC >0: #### we have more btcs than eth
+
+                        def get_btc_matching_proportion(eth_amount=None, current_price=None, lower_bound=None,
+                                                        upper_bound=None):
+                            new_liquidity = get_liquidity_from_ETH(eth_amount, current_price, upper_bound)
+                            new_btc = get_btc_from_liquidity(new_liquidity, current_price, lower_bound)
+                            return new_btc, new_liquidity
+
+                        def get_eth_matching_proportion(btc_amount=None, current_price=None, lower_bound=None,
+                                                        upper_bound=None):
+                            new_liquidity = get_liquidity_from_BTC(btc_amount, current_price, lower_bound)
+                            new_eth = get_ether_from_liquidity(new_liquidity, current_price, upper_bound)
+                            return new_eth, new_liquidity
+
+                        new_btc, _ = get_btc_matching_proportion(eth_amount=after_withdrawal_eth,
+                                                                 current_price=newly_initial_price,
+                                                                 lower_bound=new_real_lower_bound,
+                                                                 upper_bound=new_real_upper_bound)
+                        ##### we end up with eth equivalent to more btc than we have : we must swap eth
+                        has_to_swap_eth =  new_btc >= after_withdrawal_btc
+
+                        new_eth, _ = get_eth_matching_proportion(btc_amount=after_withdrawal_btc,
+                                                                 current_price=newly_initial_price,
+                                                                 lower_bound=new_real_lower_bound,
+                                                                 upper_bound=new_real_upper_bound)
+                        ##### we end up with btc equivalent to more eth than we have : we must swap btc
+                        has_to_swap_btc = new_eth >= after_withdrawal_eth
+                        print(has_to_swap_btc)
+                        print(has_to_swap_eth)
+                        if after_withdrawal_btc>0. and after_withdrawal_eth>0.:
+                            assert has_to_swap_btc == ~has_to_swap_eth
+
+                        if has_to_swap_btc: #### we have more btcs than eth
                             new_btc = after_withdrawal_btc
                             print(f'new btc {new_btc}')
                             print(f'new price {newly_initial_price}')
                             print(f'new lower bound {new_real_lower_bound}')
                             print(f'new upper bound {new_real_upper_bound}')
 
-                            def get_eth_matching_proportion(btc_amount=None, current_price=None, lower_bound=None, upper_bound=None):
-                                new_liquidity = get_liquidity_from_BTC(btc_amount, current_price,lower_bound)
-                                new_eth = get_ether_from_liquidity(new_liquidity, current_price, upper_bound)
-                                return new_eth, new_liquidity
 
-                            new_eth, _ = get_eth_matching_proportion(btc_amount = after_withdrawal_btc, current_price=newly_initial_price, lower_bound=new_real_lower_bound, upper_bound = new_real_upper_bound)
                             pool_ratio_price = new_btc/new_eth
                             print(f'new ratio {pool_ratio_price}')
 
@@ -624,23 +662,13 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                             else :
                                 print('trouble')
 
-                        elif after_withdrawal_eth >0:
-                            print(deltaETH)
-                            assert deltaETH >= 0
+                        elif has_to_swap_eth :
                             new_eth = after_withdrawal_eth
                             print(f'new eth {new_eth}')
                             print(f'new price {newly_initial_price}')
                             print(f'new lower bound {new_real_lower_bound}')
                             print(f'new upper bound {new_real_upper_bound}')
 
-                            def get_btc_matching_proportion(eth_amount=None, current_price=None, lower_bound=None, upper_bound=None):
-                                new_liquidity = get_liquidity_from_ETH(eth_amount, current_price, upper_bound)
-                                new_btc = get_btc_from_liquidity(new_liquidity, current_price, lower_bound)
-                                return new_btc, new_liquidity
-
-                            new_btc, _ = get_btc_matching_proportion(eth_amount = after_withdrawal_eth, current_price=newly_initial_price, lower_bound=new_real_lower_bound, upper_bound = new_real_upper_bound)
-                            pool_ratio_price = new_btc/new_eth
-                            print(f'new ratio {pool_ratio_price}')
 
                             def get_btc_amount_to_swap(x= None, slippage = 20e-4, initial_eth=None, initial_btc=None,lower_bound=None, upper_bound=None, current_price=None):
                                 eth_left = (1.-x)*initial_eth
@@ -700,6 +728,10 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                                 assert abs(new_liquidity-new_liquidity_bis)/abs(new_liquidity)<1e-3
                             else :
                                 print('trouble')
+                        else :
+                            new_eth = after_withdrawal_eth
+                            new_btc = after_withdrawal_btc
+
                         new_ethers[count] = new_eth
                         new_btcs[count] = new_btc
                         new_liquidities[count] = new_liquidity
@@ -747,8 +779,64 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                 else :
                     stages_rebal_df = stages_df.copy()
 
+                print('computing the mark to market value')
+
+                def compute_mark_to_market_numeraire_assets(previous_numeraire_size, previous_liquidity_size,previous_price,data_df = None):
+                    counter = 0
+                    numeraire_sizes = np.zeros(len(data_df))
+                    for i, row in data_df.iterrows():
+                        rebal = row['rebalancing']
+                        if rebal :
+                            previous_numeraire_size = row['new_btcs']
+                            previous_liquidity_size = row['new_liquidities']
+                            previous_price =  row['Close']
+                            numeraire_sizes[counter] = previous_numeraire_size
+                        else :
+                            # we mark to market the asset valo
+                            current_price = row['Close']
+                            deltanumeraire = get_new_btc_quantity_from_price_change(previous_liquidity_size, current_price, previous_price)
+                            numeraire_size = previous_numeraire_size + deltanumeraire
+                            numeraire_sizes[counter] = numeraire_size
+                        counter = counter + 1
+                    return numeraire_sizes
+                stages_rebal_df['numeraire_size_mark_to_market'] = compute_mark_to_market_numeraire_assets(to_keep_init_btc, to_keep_init_liq,to_keep_init_price,data_df=stages_rebal_df.copy())
+
+                def compute_mark_to_market_risky_assets(previous_risky_size,previous_liquidity_size,previous_price,data_df = None):
+                    counter = 0
+                    risky_sizes = np.zeros(len(data_df))
+                    for i, row in data_df.iterrows():
+                        rebal = row['rebalancing']
+                        if rebal :
+                            previous_risky_size = row['new_ethers']
+                            previous_liquidity_size = row['new_liquidities']
+                            previous_price =  row['Close']
+                            risky_sizes[counter] = previous_risky_size
+                        else :
+                            # we mark to market the asset valo
+                            current_price = row['Close']
+                            delta_risky = get_new_ether_quantity_from_price_change(previous_liquidity_size, current_price, previous_price)
+                            risky_size = previous_risky_size + delta_risky
+                            risky_sizes[counter] = risky_size
+                        counter = counter + 1
+                    return risky_sizes
+
+                stages_rebal_df['risky_size_mark_to_market'] = compute_mark_to_market_risky_assets(to_keep_init_eth,to_keep_init_liq,to_keep_init_price,data_df=stages_rebal_df.copy())
+
+                stages_rebal_df['new_btcs'] = stages_rebal_df['new_btcs'].ffill()
+                stages_rebal_df['new_ethers'] = stages_rebal_df['new_ethers'].ffill()
+                stages_rebal_df['new_liquidities'] = stages_rebal_df['new_liquidities'].ffill()
+
+                stages_rebal_df['swapping_sizes'] = stages_rebal_df['swapping_sizes'].fillna(0.)
+
+
+                stages_rebal_df['new_liquidities'] = stages_rebal_df['new_liquidities'].bfill()
+                stages_rebal_df['swapping_sizes'] = stages_rebal_df['swapping_sizes'].bfill()
+
+                stages_rebal_df = stages_rebal_df.rename(columns = {'new_btcs':'numeraire_size',
+                                                 'new_ethers':'risky_size',
+                                                 'new_liquidities':'liquidity_size'})
+
                 print('investigating the rebalancing datas')
-                rebal_data_df[['previous_btcs','previous_ethers','previous_liquidities','new_btcs','new_ethers','new_liquidities']]
 
 
                 # ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### #######
@@ -757,19 +845,13 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                 # ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### #######
                 # ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### ####### #######
 
-                all_df = pd.merge(df.copy(),  stages_rebal_df.copy(), how = 'left', left_index=True, right_index=True)
 
+
+                all_df = pd.merge(df.copy(),  stages_rebal_df.copy(),suffixes= ['','_rebal'], how = 'left', left_index=True, right_index=True)
                 all_df['swapping_cost'] = all_df['swapping_cost'].fillna(0.)
 
-                all_df['new_btcs'] = all_df['new_btcs'].ffill()
-                all_df['new_ethers'] = all_df['new_ethers'].ffill()
-                all_df['new_liquidities'] = all_df['new_liquidities'].ffill()
-                all_df['swapping_sizes'] = all_df['swapping_sizes'].ffill()
 
-                all_df['new_btcs'] = all_df['new_btcs'].bfill()
-                all_df['new_ethers'] = all_df['new_ethers'].bfill()
-                all_df['new_liquidities'] = all_df['new_liquidities'].bfill()
-                all_df['swapping_sizes'] = all_df['swapping_sizes'].bfill()
+
 
                 all_df['apr'] = all_df['apr'].fillna(0.)
                 all_df['incurred_v3'] = all_df['incurred_v3'].fillna(0.)
@@ -783,7 +865,6 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                 all_df['mark_to_market_loss_v3'] = 100. * (1.+all_df['curILv3'].values)
                 all_df['mark_to_market_loss_v2'] = 100. * (1.+all_df['curILv2'].values)
 
-                all_df['mark_to_market_lp_value'] = 100. * (1.+all_df['curlp_position_profit'].values)
 
 
                 all_df['loss_v3'] = 100. * np.cumprod(1 + all_df['incurred_v3'].values)
@@ -795,11 +876,41 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                 all_df['value_LP_LS'] = 100. * np.cumprod(1 + all_df['flipped_lp_position_profit'].values)
                 all_df['value_LP_LO'] = 100. * np.cumprod(1 + all_df['lp_position_profit'].values)
 
+                ### computing the mark to market value
+                #### curlp_position_profit is the return since the last rebalancing position : the vintage point is reseted each time we rebalance
+                ###all_df[['lp_position_profit', 'curlp_position_profit', 'rebalancing','mark_to_market_lp_value', 'value_LP_LO' ]]
+                ## this would not work weneed reset
+                all_df['mark_to_market_lp_value_without_reset'] = 100. * (1. + all_df['curlp_position_profit'].values)
+
+                def compute_lp_mark_to_market(data_df = None):
+                    previous_valo = 100.
+                    valos = np.zeros(len(data_df))
+                    counter = 0
+                    for i, row in data_df.iterrows():
+                        rebal = row['rebalancing']
+                        current_valo_since_rebal = row['curlp_position_profit']
+                        valos[counter]= previous_valo * (1. + current_valo_since_rebal)
+                        if rebal :
+                            previous_valo = valos[counter]
+                        counter = counter + 1
+                    return valos
+
+                all_df['curlp_position_profit'] = all_df['curlp_position_profit'].fillna(0.)
+                all_df['rebalancing'] = all_df['rebalancing'].fillna(False)
+
+                all_df['mark_to_market_lp_value'] = compute_lp_mark_to_market(data_df = all_df[['lp_position_profit', 'curlp_position_profit', 'rebalancing','value_LP_LO']].copy())
+                ###all_df[['lp_position_profit', 'curlp_position_profit', 'rebalancing','mark_to_market_lp_value', 'mark_to_market_lp_value_without_reset', 'value_LP_LO' ]]
+
+                all_df['mark_to_market_lp_value_returns'] = all_df['mark_to_market_lp_value'].pct_change()
+                all_df['mark_to_market_lp_value_returns'] = all_df['mark_to_market_lp_value_returns'].fillna(0.)
+
                 all_df['gain_and_value_LP_LS'] = 100. * np.cumprod(
                     1 + all_df['flipped_lp_position_profit'].values + all_df['apr'].values - all_df['swapping_cost'].values)
                 all_df['gain_and_value_LP_LO'] = 100. * np.cumprod(
                     1 + all_df['lp_position_profit'].values + all_df['apr'].values - all_df['swapping_cost'].values)
 
+                all_df['gain_and_value_LP_LO_mark_to_market'] = 100. * np.cumprod(1 + all_df['mark_to_market_lp_value_returns'].values + all_df['apr'].values - all_df['swapping_cost'].values)
+
                 #all_df['gain_and_loss_v3'] = 100. * np.cumprod(1 + all_df['incurred_v3'].values + all_df['apr'].values)
                 #all_df['gain_and_loss_v2'] = 100. * np.cumprod(1 + all_df['incurred_v2'].values + all_df['apr'].values )
 
@@ -807,99 +918,116 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                     fig = realtime_plotting_utility.plot_multiple_time_series(
                         data_df=all_df[['cumulated_swapping_costs']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'swapping costs over time{ssj}{ssj_against}')
+                        title=f'swapping costs {ssj} {ssj_against}')
                     fig.show()
                     fig = realtime_plotting_utility.plot_multiple_time_series(
                         data_df=all_df[['swapping_cost']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'swapping costs over time{ssj}{ssj_against}')
+                        title=f'swapping costs {ssj} {ssj_against}')
                     fig.show()
                     fig = realtime_plotting_utility.plot_multiple_time_series(
                         data_df=all_df[['swapping_sizes']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'swapping sizes over time{ssj}{ssj_against}')
+                        title=f'swapping sizes {ssj} {ssj_against}')
                     fig.show()
                     fig = realtime_plotting_utility.plot_multiple_time_series(
-                        data_df=all_df[['new_btcs', 'new_ethers', 'new_liquidities']].copy(), logy=False, split=False,
+                        data_df=all_df[['numeraire_size_mark_to_market','numeraire_size', 'risky_size_mark_to_market', 'risky_size']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'assets over time{ssj}{ssj_against}')
+                        title=f'assets {ssj} {ssj_against}')
                     fig.show()
                     fig = realtime_plotting_utility.plot_multiple_time_series(
-                        data_df=all_df[['new_btcs']].copy(), logy=False, split=False,
+                        data_df=all_df[['numeraire_size_mark_to_market','numeraire_size']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'BTCs amount over time{ssj}{ssj_against}')
+                        title=f'numeraire size {ssj} {ssj_against}')
                     fig.show()
                     fig = realtime_plotting_utility.plot_multiple_time_series(
-                        data_df=all_df[['new_ethers']].copy(), logy=False, split=False,
+                        data_df=all_df[['risky_size_mark_to_market','risky_size']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'ethers amount over time{ssj}{ssj_against}')
+                        title=f'risky {ssj} {ssj_against}')
                     fig.show()
                     fig = realtime_plotting_utility.plot_multiple_time_series(
-                        data_df=all_df[['new_liquidities']].copy(), logy=False, split=False,
+                        data_df=all_df[['liquidity_size']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'liquidities over time{ssj}{ssj_against}')
+                        title=f'liquidity size {ssj} {ssj_against}')
                     fig.show()
-                    fig = realtime_plotting_utility.plot_multiple_time_series(
-                        data_df=all_df[['loss_v3', 'mark_to_market_loss_v3']].copy(), logy=False, split=False,
-                        put_on_same_scale=False,
-                        title=f'v3 bounds over time{ssj}{ssj_against}')
-                    fig.show()
-                    fig = realtime_plotting_utility.plot_multiple_time_series(
-                        data_df=all_df[['loss_v3', 'mark_to_market_loss_v3']].copy(), logy=False, split=False,
-                        put_on_same_scale=False,
-                        title=f'v3 bounds over time{ssj}{ssj_against}')
-                    fig.show()
-
 
                     fig = realtime_plotting_utility.plot_multiple_time_series(
                         data_df=all_df[['loss_v3', 'mark_to_market_loss_v3']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'v3 bounds over time{ssj}{ssj_against}')
+                        title=f'loss V3 {ssj} {ssj_against}')
                     fig.show()
+
+                    fig = realtime_plotting_utility.plot_multiple_time_series(
+                        data_df=all_df[['loss_v3', 'mark_to_market_loss_v3']].copy(), logy=False, split=False,
+                        put_on_same_scale=False,
+                        title=f'loss v3 {ssj} {ssj_against}')
+                    fig.show()
+
+
                     fig = realtime_plotting_utility.plot_multiple_time_series(
                         data_df=all_df[['loss_v2', 'mark_to_market_loss_v2']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'v3 bounds over time{ssj}{ssj_against}')
+                        title=f'v3 bounds {ssj} {ssj_against}')
                     fig.show()
+
                     fig = realtime_plotting_utility.plot_multiple_time_series(
                         data_df=all_df[['value_LP_LO', 'mark_to_market_lp_value']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'v3 bounds over time{ssj}{ssj_against}')
+                        title=f'mark-to-market/book LP position value {ssj} {ssj_against}')
                     fig.show()
 
                     fig = realtime_plotting_utility.plot_multiple_time_series(
                         data_df=all_df[['real_lower_bound', 'real_upper_bound', 'Close']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'v3 bounds over time{ssj}{ssj_against}')
+                        title=f'v3 bounds {ssj} {ssj_against}')
                     fig.show()
+
                     fig = realtime_plotting_utility.plot_multiple_time_series(
                         data_df=all_df[['signal']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'v3 bounds over time{ssj}{ssj_against}')
+                        title=f'v3 bounds {ssj} {ssj_against}')
                     fig.show()
                     fig = realtime_plotting_utility.plot_multiple_time_series(
                         data_df=all_df[['curILv3']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'v3 IL over time{ssj}{ssj_against}')
+                        title=f'v3 IL {ssj} {ssj_against}')
                     fig.show()
                     fig = realtime_plotting_utility.plot_multiple_time_series(
                         data_df=all_df[['curlp_value_first']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'LP position value over time{ssj}{ssj_against}')
+                        title=f'LP position value {ssj} {ssj_against}')
                     fig.show()
                     fig = realtime_plotting_utility.plot_multiple_time_series(
                         data_df=all_df[['flipped_lp_position_profit']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'LP position value over time{ssj}{ssj_against}')
+                        title=f'LP position value {ssj} {ssj_against}')
                     fig.show()
+
                     fig1 = realtime_plotting_utility.plot_multiple_time_series(
                         #                    data_df=all_df[['loss_v2', 'loss_v3', 'gain_and_loss_v3','gain_and_loss_LS']].copy(),
                         data_df=all_df[['gain_and_value_LP_LS', 'gain_and_value_LP_LO']].copy(),
                         logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'v3 liquidity providing {ssj}{ssj_against}')
+                        title=f'Absolute performance LO/LS {ssj} {ssj_against}')
                     fig1.show()
 
+                    fig1 = realtime_plotting_utility.plot_multiple_time_series(
+                        #                    data_df=all_df[['loss_v2', 'loss_v3', 'gain_and_loss_v3','gain_and_loss_LS']].copy(),
+                        data_df=all_df[['gain_and_value_LP_LO_mark_to_market', 'gain_and_value_LP_LO']].copy(),
+                        logy=False, split=False,
+                        put_on_same_scale=False,
+                        title=f'Absolute performance v3 liquidity providing {ssj} {ssj_against}')
+                    fig1.show()
+
+                    fig1 = realtime_plotting_utility.plot_multiple_time_series(
+                        #                    data_df=all_df[['loss_v2', 'loss_v3', 'gain_and_loss_v3','gain_and_loss_LS']].copy(),
+                        data_df=all_df[['gain_and_value_LP_LO']].copy(),
+                        logy=False, split=False,
+                        put_on_same_scale=False,
+                        title=f'Absolute performance v3 liquidity providing {ssj} {ssj_against}')
+                    fig1.show()
+
+
                 stages_df = all_df.copy()
                 # computing the annulized loss
                 delta = stages_df.index[len(stages_df) - 1] - stages_df.index[0]
@@ -926,7 +1054,7 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                 on_mode = (all_df['providingLiquidity'] == 1).sum()/len(all_df)
                 from cryptotoolbox.risk_metrics import riskmetrics
 
-                kpi_df = riskmetrics.get_kpi(all_df[[f'gain_and_value_LP_LS']])
+                kpi_df = riskmetrics.get_kpi(all_df[[f'gain_and_value_LP_LO_mark_to_market']])
                 obf_data= {
                     'bb_period': bb_period,
                     'stdNbr': stdNbr,
@@ -937,7 +1065,7 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                     'v2_loss_apy': v2_loss_apy,
                     'v3_loss_apy': v3_loss_apy,
                 }
-                obf_data.update(kpi_df.to_dict()['gain_and_value_LP_LS'])
+                obf_data.update(kpi_df.to_dict()['gain_and_value_LP_LO_mark_to_market'])
 
                 objectives.append(obf_data)
 
